DECLARE @EventsOutputPath     NVARCHAR(4000);
DECLARE @EventSessionName     NVARCHAR(4000);
DECLARE @EventSessionPath     NVARCHAR(4000);
DECLARE @EventMaxFileSizeMb   INT;
DECLARE @EventFileMaxNumber   INT;
DECLARE @tsql                 NVARCHAR(MAX);
DECLARE @PathSeparator        CHAR(1);
DECLARE @LineFeed             CHAR(2);
DECLARE @MSSQLVersionMajor    INT;
DECLARE @ExecRet              INT;
DECLARE @FolderExistsTbl      TABLE (
                                FileExits       BIT,
                                isDirectory     BIT,
                                ParentDirExists BIT
                              );

-- ======================== ( User Configuration ) ==============================
                              
SET @EventsOutputPath   = 'D:\ScriptsLOGS\Monitoring\ExtendedEvents';
--SET @EventsOutputPath   = 'D:\MSSQL10_50.<InstanceName>\MSSQL\Log';
SET @EventSessionName   = 'Collect-Deadlocks';
SET @EventMaxFileSizeMb = 512;
SET @EventFileMaxNumber = 3;

-- ======================== (DO NOT EDIT FROM HERE) =============================

-- TODO: detect OS and set to slash or backslash according to Linux or Windows.
SELECT
    @PathSeparator     = CHAR(92), -- Backslash
    @LineFeed          = CHAR(13) + CHAR(10),
    @MSSQLVersionMajor = CONVERT(INT,@@MICROSOFTVERSION / 0x01000000),
    @EventsOutputPath  = CASE WHEN LEN(LTRIM(RTRIM(@EventsOutputPath))) = 0 THEN NULL ELSE @EventsOutputPath END,
    @EventSessionName  = CASE WHEN LEN(LTRIM(RTRIM(@EventSessionName))) = 0 THEN NULL ELSE @EventSessionName END
;

IF(@MSSQLVersionMajor < 10)
BEGIN
    RAISERROR('This script cannot succeed as Extended Events are available starting SQL Server 2008',12,1);
    RETURN;
END;

IF(@EventsOutputPath IS NULL)
BEGIN
    RAISERROR('Please provide events session output path',12,1);
    RETURN;
END;

IF(@EventSessionName IS NULL)
BEGIN
    RAISERROR('Please provide events session Name',12,1);
    RETURN;
END;

IF(@EventMaxFileSizeMb IS NULL OR @EventMaxFileSizeMb < 100)
BEGIN
    RAISERROR('Please provide suitable value for event session max file size', 12,1);
    RETURN;
END;

-- adding ending separator if necessary
IF(SUBSTRING(REVERSE(@EventsOutputPath),1,1) <> @PathSeparator)
BEGIN
    SET @EventsOutputPath = @EventsOutputPath + @PathSeparator
END;

-- checking path exists
INSERT INTO @FolderExistsTbl
EXEC master.dbo.xp_fileexist @EventsOutputPath
;

IF(NOT EXISTS(SELECT 1 FROM @FolderExistsTbl WHERE FileExits = 0 AND isDirectory = 1))
BEGIN
    RAISERROR('Folder [%s] does not exist on file system',12,1,@EventsOutputPath);
END;


SET @EventSessionPath = @EventsOutputPath + @EventSessionName + '.xel' ;

PRINT @EventSessionPath;

-- Building query string to create the event session
SET @tsql = 'use [master];' + @LineFeed +
            'CREATE EVENT SESSION ' + QUOTENAME(@EventSessionName) + ' ON SERVER' + @LineFeed +
            '    ADD EVENT sqlserver.xml_deadlock_report(' + @LineFeed +
            '        ACTION (' + @LineFeed +
            '            package0.collect_system_time,' + @LineFeed +
            '            sqlos.task_time,' + @LineFeed +
            '            sqlserver.client_app_name,' + @LineFeed +
            '            sqlserver.client_hostname,' + @LineFeed +
            '            sqlserver.database_id,' + @LineFeed +
            '            sqlserver.is_system,' + @LineFeed +
            '            sqlserver.username' + @LineFeed +
            '        )' + @LineFeed +
            '    )' + @LineFeed +
            '    ADD TARGET ' + CASE WHEN @MSSQLVersionMajor = 10 THEN 'package0.asynchronous_file_target' ELSE 'package0.event_file' END + '(' + @LineFeed +
            '        SET filename           = N''' + @EventSessionPath + ''',' + @LineFeed +
            '            max_file_size      = (' + CONVERT(VARCHAR(10),@EventMaxFileSizeMb) + '),' + @LineFeed +
            '            max_rollover_files = (' + CONVERT(VARCHAR(10),@EventFileMaxNumber) + ')' + @LineFeed +
            '    )' + @LineFeed +
            'WITH (' + @LineFeed +
            '     MAX_MEMORY            = 4096 KB,' + @LineFeed +
            '     EVENT_RETENTION_MODE  = ALLOW_SINGLE_EVENT_LOSS,' + @LineFeed +
            '     MAX_DISPATCH_LATENCY  = 20 SECONDS,' + @LineFeed +
            '     MAX_EVENT_SIZE        = 0 KB,' + @LineFeed +
            '     MEMORY_PARTITION_MODE = NONE,' + @LineFeed +
            '     TRACK_CAUSALITY       = OFF,' + @LineFeed +
            '     STARTUP_STATE         = ON' + @LineFeed +
            ');'
            ;

--PRINT @tsql;

IF(NOT EXISTS(SELECT 1 FROM sys.server_event_sessions where name = @EventSessionName))
BEGIN
    EXEC @ExecRet = sp_executesql @tsql;
    IF(@ExecRet <> 0)
    BEGIN
        RAISERROR('Unable to create Extended Events Session',12,1);
        RETURN;
    END;
    RAISERROR('-- Event Session [%s] created',0,1,@EventSessionName);
END;
ELSE
BEGIN
    RAISERROR('-- Event Session [%s] already exists',0,1,@EventSessionName);
END;

BEGIN TRY

    SET @tsql = 'ALTER EVENT SESSION ' + QUOTENAME(@EventSessionName) + ' ON SERVER STATE = START;';
    EXEC @ExecRet = sp_executesql @tsql;
    IF(@ExecRet <> 0)
    BEGIN
        RAISERROR('Unable to start Extended Events Session',12,1);
        RETURN;
    END;
    RAISERROR('-- Event Session [%s] started',0,1,@EventSessionName);

END TRY
BEGIN CATCH
    IF(@Debug = 1)
    BEGIN
        RAISERROR('Already started or error in configuration',0,1);
    END;
END CATCH